#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""

"""
from SLiCAP import *

Iop = 0.03 # Peak transmit current

fileName = "gainDistribution"
specFileName = "specifications.csv"

# Define an instruction
i1 = instruction()
i1.setCircuit(fileName + '.cir')

# import the specifications
specs = csv2specs(specFileName)

# Define the circuit parameters from the specifications
specs2circuit(specs, i1)

htmlPage("Circuit Data")
head2html("Circuit diagram")
img2html(fileName + ".svg", 1200)
netlist2html(fileName + ".cir")
elementData2html(i1.circuit)
params2html(i1.circuit)

htmlPage('Transmitter drive capability and gain distribution')
head2html("Determination of the transmitter transadmittance gain")
text2html("The minimum drive requirement follows from the minimum receiver noise.")
text2html("In case of an integrating receiver we minimally need:")
AR = i1.getParValue("A_Rmin")
eqn2html("A_R", AR)
i1.defPar("A_R", AR)
# Add the value of this parameter to the design specifications

specs.append(
    specItem(
        "A_R",
        description = "Minimum value of the receiver transmission coefficient $A$",
        value    = AR,
        units       = "s",
        specType    = "design",
    )
)

# Determine B_T from the system gain
i1.setSimType("numeric")
i1.setSource("V1")
i1.setDetector("V_L")
i1.setGainType('gain')
i1.setDataType("laplace")
text2html("The system gain $A_v$ is obtained as:")
Vgain = i1.execute().laplace
eqn2html("Gain", Vgain)
text2html("Its required value follows from the specifications:")
Gain = i1.getParValue("V_o")/i1.getParValue("V_i")
eqn2html("Gain", Gain)

text2html("The coefficient $B_T$ of the transmittion matrix thus needs to be:")

text2html("From this we can obtain the maximum value $B_{Tmax}$ of $B_T$:")
BTmax = sp.Abs(sp.solve(Vgain - Gain)[0])
eqn2html("B_Tmax", BTmax, units ="Omega")

# Add the value of this parameter to the design specifications

specs.append(
    specItem(
        "B_T",
        description = "Maximum value of the transmitter transmission coefficient $B$",
        value    = BTmax,
        units       = "Omega",
        specType    = "design",
    )
)

# Define the circuit parameter "B_T"

i1.defPar("B_T", BTmax)

head2html("Transmitter current drive requirement")
text2html("From the above, we find the minimum peak value $I_{oMin}$ of the drive current:")
Iomin = i1.getParValue("V_i") / BTmax
eqn2html("Io_min", Iomin)
# Add the value of the transmit coil drive current to the functional requirements
specs.append(
    specItem(
        "Io_p",
        description = "Minimum peak drive current of the transmit coil",
        value    = Iomin,
        units       = "A",
        specType    = "design",
    )
)

head2html("Transmitter voltage drive capability")
text2html("""The voltage drive requirement of the transmitter is found by
          multiplying the peak voltage of the source with the voltage gain from
          the source to the voltage across the transmit coil, at the largest
          full-power frequency and add it to the input voltage,""")

i1.setSimType("symbolic")
laplaceViResult = i1.execute()
VTRcoil = sp.sqrt(2) * sp.Symbol("V_i") * sp.Abs(laplaceViResult.laplace.subs(ini.Laplace, 2*sp.I*sp.pi*sp.Symbol('f_fpl')))
eqn2html("V_TRcoil", VTRcoil)
VTRcoilNum = sp.N(fullSubs(VTRcoil, i1.parDefs))
eqn2html("V_TRcoil", VTRcoilNum)

text2html("""At the highest frequency the voltage across the coil has 90 degrees
          phase shift with respect to the input voltage. The total voltage
          $V_{TRp}$ at the tramnsmitter output thus equals:""")

VTRp = sp.N(sp.sqrt(VTRcoilNum**2 + i1.getParValue("V_i")**2))
eqn2html("V_TRp", VTRp)

specs.append(
    specItem(
        "V_TRp",
        description = "Peak transmitter voltage",
        value    = VTRp,
        units       = "V",
        specType    = "design",
    )
)

text2html("The voltage slew rate $SR_{vT}$ of the transadmittance transmittance " +
          "amplifier should minimally be:")

SRvT = sp.pi*2*sp.Symbol('f_fpl')*VTRp
eqn2html("SR_vT",SRvT)

text2html("After substitution of all the known variables, we obtain:")
SRvTnum = sp.N(fullSubs(SRvT, i1.parDefs))
eqn2html("SR_vT",SRvTnum)
# Add the transmitter voltage drive requirements requirement to the design specifications
specs.append(
    specItem(
        "V_TRcoil",
        description = "Transmitter peak voltage drive capability",
        value    = VTRcoilNum,
        units       = "V",
        specType    = "design",
    )
)
specs.append(
    specItem(
        "SR_vT",
        description = "Transmitter voltage slew rate",
        value    = SRvTnum,
        units       = "V/s",
        specType    = "design",
    )
)

head2html("Receiver peak input voltage")
text2html("The peak input voltage $V_{rec}$ of the receiver is found by " +
          "taking the voltage at node 'o' as detector voltage.")
i1.setDetector("V_o")
Vrec = sp.sqrt(2) * sp.Symbol("V_i") * sp.Abs(i1.execute().laplace.subs(ini.Laplace, 2*sp.pi*sp.I*sp.Symbol("f_fpl")))
eqn2html("V_rec", Vrec)
Vrec = sp.N(fullSubs(Vrec, i1.parDefs))
eqn2html("V_rec", Vrec)

# Add the transmitter voltage drive requirement requirement to the design specifications
specs.append(
    specItem(
        "V_rec",
        description = "Peak receiver input voltage",
        value    = Vrec,
        units       = "V",
        specType    = "design",
    )
)

head2html("Receiver voltage drive capability")
text2html("The voltage drive capability of the receiver is specified in the " +
          "application description ($V_o$).")

text2html("The voltage slew-rate of the receiver amplifier should minimally be:")
SRvR = sp.pi*2*sp.Symbol('f_fpl')*sp.Symbol("V_o")
eqn2html("SR_vR", SRvR)
text2html("After substitution of all the known variables, we obtain:")
SRvRnum = sp.N(fullSubs(SRvR, i1.parDefs))
eqn2html("SR_vR", SRvRnum)
head2html("Receiver current drive capability")
text2html("The current-drive capability $I_L$ of the receiver should minimally be:")
IL = sp.Symbol("V_o")/sp.Symbol("R_ell")
eqn2html("I_L", IL)
text2html("After substitution of all the known variables, we obtain:")
ILnum = sp.N(fullSubs(IL, i1.parDefs))
eqn2html("I_L", ILnum)

# Add the value of the voltage gain to the design specifications
specs.append(
    specItem(
        "I_L",
        description = "Receiver current drive capability",
        value    = ILnum,
        units       = "A",
        specType    = "design",
    )
)
specs.append(
    specItem(
        "SR_vR",
        description = "Receiver voltage slew rate",
        value    = SRvRnum,
        units       = "V/s",
        specType    = "design",
    )
)

# Save the specifications
specs2csv(specs, "specifications.csv")

# Display the specifications on a html page.
htmlPage("Updated specifications")
specs2html(specs)